/**************************************************************************
*                                                                         *
*   EthernetMIDI - multiplatform MIDI over Ethernet / LAN                 *
*                                                                         *
*   Copyright (C) 2005 by Benno Senoner , Italy                           *
*   http://www.linuxsampler.org/ethernetmidi                              *
*   EMail: sbenno@gardena.net                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the Free Software           *
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
*   MA  02111-1307  USA                                                   *
***************************************************************************/

#include "mainwindow.h"

#include <stdio.h>
#include <math.h>

#include <qlabel.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qtextedit.h>
#include <qbuttongroup.h>



MainWindow::MainWindow(QWidget* parent, const char* name, WFlags fl) : MainWindowBase(parent,name,fl)  {
printf("MainWindow constructor !\n");
  resize(530,560);


  RemotePortLineEdit->setText("9000");
  LocalPortLineEdit->setText("9000");

  MidiEnabled = false;

  InputPortIndex = 0;
  OutputPortIndex = 0;

  MidiInActivity = 0;
  MidiOutActivity = 0;

  socketio = new SocketIO();
  socketio->init();

  midiio = new MidiIO(socketio, &MidiInActivity);

  QStringList stringlist;
  midiio->getMidiInPorts(stringlist);

  MidiInListBox->setAutoBottomScrollBar(false);
  MidiInListBox->clear();
  for( QStringList::Iterator it = stringlist.begin(); it != stringlist.end(); ++it ) {
    MidiInListBox->insertItem(*it);
  }


  stringlist.clear();

  midiio->getMidiOutPorts(stringlist);

  MidiOutListBox->setAutoBottomScrollBar(false);

  MidiOutListBox->clear();
  for( QStringList::Iterator it = stringlist.begin(); it != stringlist.end(); ++it ) {
    MidiOutListBox->insertItem(*it);
  }

  connect(MidiInListBox, SIGNAL(highlighted(int)), this, SLOT(midiin_listbox_highlighted(int)) );
  connect(MidiOutListBox, SIGNAL(highlighted(int)), this, SLOT(midiout_listbox_highlighted(int)) );
  connect(StartStopButtonGroup, SIGNAL(clicked(int)), this, SLOT(startstop_buttongroup_clicked(int)) );

  QTimer *mytimer;
  mytimer = new QTimer(this);
  mytimer->start(200);
  
  connect(mytimer, SIGNAL(timeout()), this, SLOT(timer_slot()) );

  // start the midi thread
  midithread = new MidiThread(this);
  //midithread->start(QThread::TimeCriticalPriority);
  midithread->start(QThread::HighestPriority);
  
}

MainWindow::~MainWindow() {
printf("MainWindow destructor !\n");
}


void MainWindow::logMessage(const QString &str, int mode) {
  QString logstr;
  QTime currTime = QTime::currentTime();
  logstr.sprintf("%2.2d:%2.2d:%2.2d", currTime.hour(), currTime.minute(), currTime.second() );
  logstr += "  " + str;
  LogTextEdit->append(logstr);
  LogTextEdit->scrollToBottom();
}

void MainWindow::startstop_buttongroup_clicked(int id) {
  int res;
  int port;
  int local_port;
  QString str;
  switch(id) {

    case 0:
      if(MidiEnabled == true) break;
      res = midiio->openMidiIn(InputPortIndex);
      if(res == 0) {
        str.sprintf("Opened MIDI IN port %d", InputPortIndex);
        logMessage(str, 0);
      }
      else {
        str.sprintf("Failed to open MIDI IN port %d", InputPortIndex);
        logMessage(str, 0);
        break;
      }

// FIXME: remove the comments, check if openMidiOut does need
// to be called in the MidiThread. midi functions not thread safe ?
      res = midiio->openMidiOut(OutputPortIndex);
      if(res == 0) {
        str.sprintf("Opened MIDI OUT port %d", OutputPortIndex);
        logMessage(str, 0);
      }
      else {
        str.sprintf("Failed to open MIDI OUT port %d", OutputPortIndex);
        logMessage(str, 0);
        midiio->closeMidiIn();
        str.sprintf("Closed MIDI IN port");
        logMessage(str, 0);
        break;
      }

      port = RemotePortLineEdit->text().toUShort();
      local_port = LocalPortLineEdit->text().toUShort();
// FIXME: remove below
//      str.sprintf("calling openUdpSocket: hostname='%s' port=%d local_port=%d",(const char *)RemoteIPAddressLineEdit->text(), port, local_port);
//      logMessage(str, 0);

      res = socketio->openUdpSocket( (const char *)RemoteIPAddressLineEdit->text(), port, local_port);

      if(res == 0) {
        str.sprintf("Opened UDP network socket");
        logMessage(str, 0);
      }
      else {
        // FIXME: add error handling
      }

      MidiEnabled = true;
      break;

    case 1:
      if(MidiEnabled == false) break;
      MidiEnabled = false;
      midiio->closeMidiIn();
      str.sprintf("Closed MIDI IN port");
      logMessage(str, 0);
      midiio->closeMidiOut();
      str.sprintf("Closed MIDI OUT port");
      logMessage(str, 0);

      socketio->closeUDPSocket();
      str.sprintf("Closed UDP network socket");
      logMessage(str, 0);

      break;
  }
  
}


void MainWindow::midiin_listbox_highlighted(int id) {
  InputPortIndex = id;
}

void MainWindow::midiout_listbox_highlighted(int id) {
  OutputPortIndex = id;
}


void MainWindow::timer_slot(void) {
  //InputPortIndex++;
  //LocalPortLineEdit->setText(QString::number(InputPortIndex));

  // FIXME: remove below, only for testing
  TestTextLabel->hide();
  TestTextLabel->setNum(testvalue);

  switch(MidiInActivity) {
    case 1:
      OutgoingMidiFrame->setPaletteBackgroundColor(Qt::green);
      MidiInActivity = 0;
      break;

    case 0:
      OutgoingMidiFrame->setPaletteBackgroundColor(Qt::gray);
      MidiInActivity = 2;
      break;
    default:
      break;
  }

  switch(MidiOutActivity) {
    case 1:
      IncomingMidiFrame->setPaletteBackgroundColor(Qt::green);
      MidiOutActivity = 0;
      break;

    case 0:
      IncomingMidiFrame->setPaletteBackgroundColor(Qt::gray);
      MidiOutActivity = 2;
      break;
    default:
      break;
  }


}

