/**************************************************************************
*                                                                         *
*   EthernetMIDI - multiplatform MIDI over Ethernet / LAN                 *
*                                                                         *
*   Copyright (C) 2005 by Benno Senoner , Italy                           *
*   http://www.linuxsampler.org/ethernetmidi                              *
*   EMail: sbenno@gardena.net                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the Free Software           *
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
*   MA  02111-1307  USA                                                   *
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>


//#include <windows.h>
//#include <mmsystem.h>

#include "midiio.h"

typedef union {
          DWORD dword;
          char data[4];
        } midi_data_t;


static void CALLBACK win32_midiin_callback(HMIDIIN handle, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2) {
// FIXME: remote this is to test if it crashed
int *a;
a = NULL;
//if(uMsg == MIM_DATA) *a=10;

  MidiIO *m = (MidiIO *)dwInstance;
  m->win32MidiInCallback(handle, uMsg, dwParam1, dwParam2);
}

void MidiIO::win32MidiInCallback(HMIDIIN handle, UINT uMsg, DWORD dwParam1, DWORD dwParam2) {

  midi_data_t midi_data;
  midi_data.dword=dwParam1;


  switch(uMsg) {
    case MIM_DATA:
      //printf("MIM_DATA: %2.2x %2.2x %2.2x\n",midi_data.data[0], midi_data.data[1], midi_data.data[2]);
// FIXME: send the midi 4 byte dword as is
// need to change the metod and parse the midi stream to make
// variable sized network packets
      socketio->udpSend(&midi_data.dword, 4);
      *MidiInActivity = 1; // this causes the GUI MIDI IN LED to go green for a moment
      break;
    default:
      //printf("callback: uMsg=%d\n",uMsg);
      break;
  }
  //fflush(stdout);

}


MidiIO::MidiIO(SocketIO *socketio_ptr, int *midiin_activity_ptr) {

  MidiInOpened = false;
  MidiOutOpened = false;

  socketio = socketio_ptr;

  MidiInActivity = midiin_activity_ptr;
  
}

MidiIO::~MidiIO() {
}



int MidiIO::getMidiInPorts(QStringList &stringlist) {

  MIDIINCAPS midiincaps;
  QString str;
  char s[200];

  int NumDevs = midiInGetNumDevs();

  for(int i=0;i<NumDevs;i++) {
    int res = midiInGetDevCaps(i, &midiincaps, sizeof(MIDIINCAPS));
    if(res == MMSYSERR_NOERROR) {

      int u;
      for(u=0; midiincaps.szPname[u] != 0; u++) {
        s[u] = midiincaps.szPname[u];
      }
      s[u]=0;
      str.sprintf("%s",(const char *)s);
      stringlist += str;
    }
  }
  return NumDevs;
}


int MidiIO::getMidiOutPorts(QStringList &stringlist) {

  MIDIOUTCAPS midioutcaps;
  QString str;
  char s[200];

  int NumDevs = midiOutGetNumDevs();

  for(int i=0;i<NumDevs;i++) {
    int res = midiOutGetDevCaps(i, &midioutcaps, sizeof(MIDIOUTCAPS));
    if(res == MMSYSERR_NOERROR) {

      int u;
      for(u=0; midioutcaps.szPname[u] != 0; u++) {
        s[u] = midioutcaps.szPname[u];
      }
      s[u]=0;
      str.sprintf("%s",(const char *)s);
      stringlist += str;
    }
  }
  return NumDevs;
}



int MidiIO::openMidiIn(int device_id) {

  MMRESULT res;

/*
   FIXME: is midiInOpen 64bit clean ?  (eg how to make it 64bit Windows compatible ?)
   The 4th argument (dwInstance) is only
   a DWORD (32bit) but we need to use it as a pointer to the MidiIO::this
   in order to let win32_midiin_callback call the MidiIO:: call back function
*/
  res = midiInOpen(&inhandle, device_id, (DWORD)win32_midiin_callback, (DWORD)this, CALLBACK_FUNCTION);
  if(res != MMSYSERR_NOERROR) {
    //printf("midiInOpen error !\n");
    return(-1);
  }

  midihdr.lpData = (CHAR *)&SysExBuffer[0];
  midihdr.dwBufferLength = sizeof(SysExBuffer);

  res = midiInPrepareHeader(inhandle, &midihdr, sizeof(MIDIHDR));
  if(res != MMSYSERR_NOERROR) {
    //printf("midiInPrepareHeader error !\n");
    midiInClose(inhandle);
    return(-1);
  }

  res = midiInStart(inhandle);
  if(res != MMSYSERR_NOERROR) {
    //printf("midiInStart error !\n");
    midiInClose(inhandle);
    return(-1);
  }

  MidiInOpened = true;
  return 0;
}


int MidiIO::openMidiOut(int device_id) {

  MMRESULT res;

  res = midiOutOpen(&outhandle, device_id, 0, 0, CALLBACK_NULL);
  if(res != MMSYSERR_NOERROR) {
    //printf("midiOutOpen error !\n");
    return(-1);
  }

  MidiOutOpened = true;
  return 0;
}


void MidiIO::closeMidiIn() {
  if(MidiInOpened == true) {
    midiInClose(inhandle);
    MidiInOpened = false;
  }
}


void MidiIO::closeMidiOut() {
  if(MidiOutOpened == true) {
    midiOutClose(outhandle);
    MidiOutOpened = false;
  }
}


int MidiIO::sendMidiData(unsigned char *buffer, int len) {
// FIXME: replace midiOutShortMsg with a general function that can
// output arbitrary midi data (including sysex)
DWORD mydword;
memcpy(&mydword, buffer, 4);
    midiOutShortMsg(outhandle, mydword);
    //midiOutShortMsg(outhandle, 0x00403C90);

  return 0;
}



void testmain(int argc, char **argv) {

//FIXME: sysexbuffer
unsigned char SysExBuffer[256];

  HMIDIOUT outhandle;
  HMIDIIN inhandle;

  MIDIHDR midihdr;

  MMRESULT res;

  MIDIINCAPS midiincaps;
  unsigned long NumInputDevs, i;

  if(argc != 2) {
    //printf("usage: mytest input(output)\n");
    exit(0);
  }

  if(!strcmp(argv[1],"input")) {

    NumInputDevs = midiInGetNumDevs();
    //printf("Number of MIDI Input devices =%d\n", NumInputDevs);

    for(i=0;i<NumInputDevs;i++) {
      res = midiInGetDevCaps(i, &midiincaps, sizeof(MIDIINCAPS));
      if(res == MMSYSERR_NOERROR) {
        //printf("DEVICE: id=%d name='%s'\n",i, midiincaps.szPname);
      }
      
    }



    //res = midiInOpen(&inhandle, 0, (DWORD)midiCallBack, 0, CALLBACK_FUNCTION);
    if(res != MMSYSERR_NOERROR) {
      //printf("midiInOpen error !\n");
      exit(0);
    }

//    midihdr.lpData = (unsigned char *)&SysExBuffer[0];
    midihdr.lpData = (CHAR *)&SysExBuffer[0];
    midihdr.dwBufferLength = sizeof(SysExBuffer);

    res = midiInPrepareHeader(inhandle, &midihdr, sizeof(MIDIHDR));
    if(res != MMSYSERR_NOERROR) {
      //printf("midiInPrepareHeader error !\n");
      exit(0);
    }

    res = midiInStart(inhandle);
    if(res != MMSYSERR_NOERROR) {
      //printf("midiInStart error !\n");
      exit(0);
    }


    Sleep(100000);


    midiInClose(inhandle);
    
  }


  if(!strcmp(argv[1],"output")) {
   if (!midiOutOpen(&outhandle, 1, 0, 0, CALLBACK_NULL) ) {
    /* Output the C note (ie, sound the note) */
    midiOutShortMsg(outhandle, 0x00403C90);

    /* Output the E note */
    midiOutShortMsg(outhandle, 0x00404090);

    /* Output the G note */
    midiOutShortMsg(outhandle, 0x00404390);

    /* Here you should insert a delay so that you can hear the notes sounding */
    Sleep(1000);

    /* Now let's turn off those 3 notes */
    midiOutShortMsg(outhandle, 0x00003C90);
    midiOutShortMsg(outhandle, 0x00004090);
    midiOutShortMsg(outhandle, 0x00004390);

    Sleep(5000);
     /* Close the MIDI device */
     midiOutClose(outhandle);
    }
  }

}

