/**************************************************************************
*                                                                         *
*   EthernetMIDI - multiplatform MIDI over Ethernet / LAN                 *
*                                                                         *
*   Copyright (C) 2005 by Benno Senoner , Italy                           *
*   http://www.linuxsampler.org/ethernetmidi                              *
*   EMail: sbenno@gardena.net                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the Free Software           *
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
*   MA  02111-1307  USA                                                   *
***************************************************************************/


#include "socketio.h"


SocketIO::SocketIO() {
//  m_socket = -1;
//  udp_socket = -1;
}


SocketIO::~SocketIO() {
  WSACleanup();
}

int SocketIO::init(void) {
  int res = WSAStartup(MAKEWORD(2,2), &wsaData);
  if(res != NO_ERROR) return(-1);
  return(0);
}


int SocketIO::openTCPSocket(const char *hostname, unsigned short port) {
    
    m_socket = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );

    if ( m_socket == INVALID_SOCKET ) {
        //printf( "Error at socket(): %ld\n", WSAGetLastError() );
        WSACleanup();
        return(-1);
    }

    // Connect to a server.
    sockaddr_in clientService;

    clientService.sin_family = AF_INET;
    clientService.sin_addr.s_addr = inet_addr( hostname );
    clientService.sin_port = htons( port );

    if ( connect( m_socket, (SOCKADDR*) &clientService, sizeof(clientService) ) == SOCKET_ERROR) {
      //   printf( "Failed to connect.\n" );
      WSACleanup();
      return(-1);
    }
    //printf("socket connected\n");
    //fflush(stdout);

  return(0);
}

int SocketIO::openUdpSocket(const char *hostname, unsigned short port, unsigned short local_port) {

  struct addrinfo *aiList = NULL;

  struct addrinfo aiHints;

  char port_str[10];
  sprintf(port_str,"%d", port);
printf("port_str='%s'\n",port_str);
  
    
  udp_socket = socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );

  if ( udp_socket == INVALID_SOCKET ) {
    //printf( "Error at socket(): %ld\n", WSAGetLastError() );
    WSACleanup();
    return(-1);
  }

  if(port > 0) {

    aiHints.ai_family = AF_INET;
    aiHints.ai_socktype = SOCK_DGRAM;
    aiHints.ai_protocol = IPPROTO_UDP;
  

    int res = getaddrinfo(hostname, NULL, NULL, &aiList);

    printf("getaddriinfo res=%d\n",res);
    if(res != 0) {
      return(-1);
    }

    sockaddr_in *temp_sockaddr_in = (sockaddr_in *)aiList->ai_addr;
    //  RecvAddr.sin_addr.s_addr = temp_sockaddr_in->sin_addr.s_addr;

    memcpy(&RemoteAddr, temp_sockaddr_in, sizeof(RemoteAddr));
    RemoteAddr.sin_family = AF_INET;
    RemoteAddr.sin_port = htons(port);

    printf("RemoteAddr.sin_addr.s_addr=%x\n", RemoteAddr.sin_addr.s_addr);
  }

  if(local_port > 0) {
    memset(&LocalAddr, 0, sizeof(LocalAddr));
    LocalAddr.sin_family = AF_INET;
    LocalAddr.sin_port = htons(local_port);
    LocalAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    bind(udp_socket, (SOCKADDR *) &LocalAddr, sizeof(LocalAddr));
  }


  
  return(0);
}

int SocketIO::udpSend(void *buffer, int len) {
  int res = sendto(udp_socket, (char *)buffer, len, 0, (SOCKADDR *) &RemoteAddr, sizeof(RemoteAddr));
  return res;
}

int SocketIO::udpRecv(void *buffer, int len) {
  int remote_addr_len = sizeof(RemoteAddr);
  int res = recvfrom(udp_socket, (char *)buffer, len, 0, (SOCKADDR *) &SenderAddr, &remote_addr_len);
  return res;
}


int SocketIO::send(void *buffer, int len, int flags) {

  int bytesSent;
  bytesSent = ::send( m_socket, (char *)buffer, len, 0 );
  return(bytesSent);
}

int SocketIO::recv(void *buffer, int len, int flags) {

  int bytesRecv;
  bytesRecv = ::recv( m_socket, (char *)buffer, len, 0 );
  return(bytesRecv);
}

int SocketIO::closeTCPSocket() {
  closesocket(m_socket);
  return 0;
}

int SocketIO::closeUDPSocket() {
  closesocket(udp_socket);
  return 0;
}

